@@ -20,6 +20,7 @@ from keystoneauth1 import adapter
 from keystoneauth1 import discover
 from keystoneauth1 import exceptions
 from keystoneauth1 import fixture
+from keystoneauth1 import http_basic
 from keystoneauth1 import noauth
 from keystoneauth1 import session
 from keystoneauth1.tests.unit import utils
@@ -747,24 +748,81 @@ class VersionDataTests(utils.TestCase):
 
     def test_endpoint_data_noauth_discover(self):
         mock = self.requests_mock.get(
+            BASE_URL, status_code=200, json=V3_VERSION_LIST)
+        self.requests_mock.get(
             V3_URL, status_code=200, json=V3_VERSION_ENTRY)
+
+        plugin = noauth.NoAuth(endpoint=BASE_URL)
+        data = plugin.get_endpoint_data(self.session)
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(plugin.get_api_major_version(self.session), (3, 0))
+        self.assertEqual(plugin.get_endpoint(self.session), BASE_URL)
+
+        self.assertTrue(mock.called_once)
+
+    def test_endpoint_data_noauth_versioned_discover(self):
+        self.requests_mock.get(
+            BASE_URL, status_code=200, json=V3_VERSION_LIST)
+        self.requests_mock.get(
+            V3_URL, status_code=200, json=V3_VERSION_ENTRY)
+
+        plugin = noauth.NoAuth(endpoint=V3_URL)
+        data = plugin.get_endpoint_data(self.session)
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(plugin.get_api_major_version(self.session), (3, 0))
+        self.assertEqual(plugin.get_endpoint(self.session), V3_URL)
+
+    def test_endpoint_data_noauth_no_discover(self):
+        plugin = noauth.NoAuth(endpoint=V3_URL)
+        data = plugin.get_endpoint_data(
+            self.session, discover_versions=False)
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(plugin.get_api_major_version(self.session), (3, 0))
+        self.assertEqual(plugin.get_endpoint(self.session), V3_URL)
+
+    def test_endpoint_data_noauth_override_no_discover(self):
         plugin = noauth.NoAuth()
-        data = plugin.get_endpoint_data(self.session, endpoint_override=V3_URL)
+        data = plugin.get_endpoint_data(
+            self.session, endpoint_override=V3_URL, discover_versions=False)
 
         self.assertEqual(data.api_version, (3, 0))
         self.assertEqual(data.url, V3_URL)
-        self.assertEqual(
-            plugin.get_api_major_version(
-                self.session, endpoint_override=V3_URL),
-            (3, 0))
         self.assertEqual(
             plugin.get_endpoint(self.session, endpoint_override=V3_URL),
             V3_URL)
 
-        self.assertTrue(mock.called_once)
+    def test_endpoint_data_http_basic_discover(self):
+        self.requests_mock.get(
+            BASE_URL, status_code=200, json=V3_VERSION_LIST)
+        self.requests_mock.get(
+            V3_URL, status_code=200, json=V3_VERSION_ENTRY)
 
-    def test_endpoint_data_noauth_no_discover(self):
-        plugin = noauth.NoAuth()
+        plugin = http_basic.HTTPBasicAuth(endpoint=V3_URL)
+        data = plugin.get_endpoint_data(self.session)
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(plugin.get_api_major_version(self.session), (3, 0))
+        self.assertEqual(plugin.get_endpoint(self.session), V3_URL)
+
+    def test_endpoint_data_http_basic_no_discover(self):
+        plugin = http_basic.HTTPBasicAuth(endpoint=V3_URL)
+        data = plugin.get_endpoint_data(
+            self.session, discover_versions=False)
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(plugin.get_api_major_version(self.session), (3, 0))
+        self.assertEqual(plugin.get_endpoint(self.session), V3_URL)
+
+    def test_endpoint_data_http_basic_override_no_discover(self):
+        plugin = http_basic.HTTPBasicAuth()
         data = plugin.get_endpoint_data(
             self.session, endpoint_override=V3_URL, discover_versions=False)
 
@@ -779,6 +837,22 @@ class VersionDataTests(utils.TestCase):
             V3_URL)
 
     def test_endpoint_data_noauth_adapter(self):
+        self.requests_mock.get(
+            BASE_URL, status_code=200, json=V3_VERSION_LIST)
+        self.requests_mock.get(
+            V3_URL, status_code=200, json=V3_VERSION_ENTRY)
+
+        client = adapter.Adapter(
+            session.Session(noauth.NoAuth()),
+            endpoint_override=BASE_URL)
+        data = client.get_endpoint_data()
+
+        self.assertEqual(data.api_version, (3, 0))
+        self.assertEqual(data.url, V3_URL)
+        self.assertEqual(client.get_api_major_version(), (3, 0))
+        self.assertEqual(client.get_endpoint(), BASE_URL)
+
+    def test_endpoint_data_noauth_versioned_adapter(self):
         mock = self.requests_mock.get(
             V3_URL, status_code=200, json=V3_VERSION_ENTRY)
 
@@ -1222,6 +1296,10 @@ class VersionDataTests(utils.TestCase):
 
 
 class EndpointDataTests(utils.TestCase):
+    def setUp(self):
+        super(EndpointDataTests, self).setUp()
+        self.session = session.Session()
+
     @mock.patch('keystoneauth1.discover.get_discovery')
     @mock.patch('keystoneauth1.discover.EndpointData.'
                 '_get_discovery_url_choices')
@@ -1285,3 +1363,12 @@ class EndpointDataTests(utils.TestCase):
         bad_url = "https://compute.example.com/v2/123456"
         epd = discover.EndpointData(catalog_url=bad_url)
         self.assertEqual((2, 0), epd.api_version)
+
+    def test_url_version_match_project_id_int(self):
+        self.session = session.Session()
+        discovery_fixture = fixture.V3Discovery(V3_URL)
+        discovery_doc = _create_single_version(discovery_fixture)
+        self.requests_mock.get(V3_URL, status_code=200, json=discovery_doc)
+        epd = discover.EndpointData(catalog_url=V3_URL).get_versioned_data(
+            session=self.session, project_id='3')
+        self.assertEqual(epd.catalog_url, epd.url)
